@@ -19,8 +19,9 @@ cacheMetaReqUpdate <- function(cacheFileMetaVer, surveyName=NULL){
   }
   
   surveyDef <- c("NAEP", #NAEP Dataset
-               "TIMSS", "TIMSS Advanced", "PIRLS", "ICILS", "ICCS", "CivED", #IEA Datasets
-               "PIAAC", "PISA", "TALIS") #OECD Datasets
+               "TIMSS", "TIMSS Advanced", "PIRLS", "ePIRLS", "ICILS", "ICCS", "CivED", #IEA Datasets
+               "PIAAC", "PISA", "TALIS", #OECD Datasets
+               "ECLS_K", "ECLS_K2011", "ELS", "HSLS") #NCES longitudinal Dataset
   
   #build our lookup table
   surveyLookup <- data.frame(survey=surveyDef, cacheVer=vector("integer", length(surveyDef)), stringsAsFactors = FALSE)
@@ -30,12 +31,18 @@ cacheMetaReqUpdate <- function(cacheFileMetaVer, surveyName=NULL){
   surveyLookup[surveyLookup$survey=="TIMSS", "cacheVer"] <- 3
   surveyLookup[surveyLookup$survey=="TIMSS Advanced", "cacheVer"] <- 2
   surveyLookup[surveyLookup$survey=="PIRLS", "cacheVer"] <- 3
+  surveyLookup[surveyLookup$survey=="ePIRLS", "cacheVer"] <- 1
   surveyLookup[surveyLookup$survey=="ICILS", "cacheVer"] <- 2
   surveyLookup[surveyLookup$survey=="ICCS", "cacheVer"] <- 3
   surveyLookup[surveyLookup$survey=="CivED", "cacheVer"] <- 5
   surveyLookup[surveyLookup$survey=="PIAAC", "cacheVer"] <- 3
   surveyLookup[surveyLookup$survey=="PISA", "cacheVer"] <- 4
   surveyLookup[surveyLookup$survey=="TALIS", "cacheVer"] <- 4
+  surveyLookup[surveyLookup$survey=="ECLS_K", "cacheVer"] <- 1
+  surveyLookup[surveyLookup$survey=="ECLS_K2011", "cacheVer"] <- 1
+  surveyLookup[surveyLookup$survey=="ELS", "cacheVer"] <- 1
+  surveyLookup[surveyLookup$survey=="HSLS", "cacheVer"] <- 1
+  surveyLookup[surveyLookup$survey=="ECLS_B", "cacheVer"] <- 1
   
   if(!any(surveyLookup$survey %in% surveyName)){
     warning("Survey name not recognized while checking .meta version. Forcing cache .meta file to be updated.")
@@ -44,11 +51,6 @@ cacheMetaReqUpdate <- function(cacheFileMetaVer, surveyName=NULL){
   
   testVal <- surveyLookup[surveyLookup$survey==surveyName, "cacheVer"]
   
-  if(cacheFileMetaVer < testVal){ #test the .meta cache version vs the versin specified
-    cacheMetaReqUpdate <- TRUE
-  } else {
-    cacheMetaReqUpdate <- FALSE
-  }
-  
-  return(cacheMetaReqUpdate)
+  #test the .meta cache version vs the versin specified
+  return(cacheFileMetaVer < testVal)
 }
